@@ -1368,6 +1368,88 @@ async def test_all_same_exception_and_with_different_errno(
     ]
 
 
+@patch_socket
+@pytest.mark.asyncio
+@pytest.mark.xfail(reason="raises RuntimeError: coroutine ignored GeneratorExit")
+async def test_handling_system_exit(
+    m_socket: ModuleType,
+) -> None:
+    """Test handling SystemExit."""
+    mock_socket = mock.MagicMock(
+        family=socket.AF_INET,
+        type=socket.SOCK_STREAM,
+        proto=socket.IPPROTO_TCP,
+        fileno=mock.MagicMock(return_value=1),
+    )
+    create_calls = []
+
+    def _socket(*args, **kw):
+        for attr in kw:
+            setattr(mock_socket, attr, kw[attr])
+        return mock_socket
+
+    async def _sock_connect(
+        sock: socket.socket, address: Tuple[str, int, int, int]
+    ) -> None:
+        create_calls.append(address)
+        raise SystemExit
+
+    m_socket.socket = _socket  # type: ignore
+    ipv6_addr_info = (
+        socket.AF_INET6,
+        socket.SOCK_STREAM,
+        socket.IPPROTO_TCP,
+        "",
+        ("dead:beef::", 80, 0, 0),
+    )
+    ipv6_addr_info_2 = (
+        socket.AF_INET6,
+        socket.SOCK_STREAM,
+        socket.IPPROTO_TCP,
+        "",
+        ("dead:aaaa::", 80, 0, 0),
+    )
+    ipv4_addr_info = (
+        socket.AF_INET,
+        socket.SOCK_STREAM,
+        socket.IPPROTO_TCP,
+        "",
+        ("107.6.106.83", 80),
+    )
+    addr_info = [ipv6_addr_info, ipv6_addr_info_2, ipv4_addr_info]
+    local_addr_infos = [
+        (
+            socket.AF_INET6,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("::1", 0, 0, 0),
+        ),
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("127.0.0.1", 0),
+        ),
+    ]
+    loop = asyncio.get_running_loop()
+    with pytest.raises(SystemExit), mock.patch.object(
+        loop, "sock_connect", _sock_connect
+    ):
+        await start_connection(
+            addr_info,
+            happy_eyeballs_delay=0.3,
+            interleave=2,
+            local_addr_infos=local_addr_infos,
+        )
+
+    # Stopped after the first call
+    assert create_calls == [
+        ("dead:beef::", 80, 0, 0),
+    ]
+
+
 @pytest.mark.asyncio
 @pytest.mark.skipif(sys.version_info >= (3, 8, 2), reason="requires < python 3.8.2")
 def test_python_38_compat() -> None:
